"use strict";

var Helper = require('../Helper');

require("../mocha-config.js");

var AliasMarker = require('../../../main/js/map/marker/AliasMarker');
var AliasOverlay = require('../../../main/js/map/overlay/AliasOverlay');
var CustomMap = require('../../../main/js/map/CustomMap');
var IdentifiedElement = require('../../../main/js/map/data/IdentifiedElement');
var OverlayCollection = require('../../../main/js/map/overlay/OverlayCollection');
var ReactionOverlay = require('../../../main/js/map/overlay/ReactionOverlay');

var logger = require('./../logger');

var chai = require('chai');
var assert = chai.assert;

describe('CustomMap', function() {

  var helper;
  before(function() {
    helper = new Helper();
  });

  it("contructor", function() {
    var options = helper.createCustomMapOptions();
    var map = new CustomMap(options);
    assert.ok(map);
  });

  it("contructor", function() {
    var options = helper.createCustomMapOptions();
    var map = new CustomMap(options);
    assert.ok(map);
  });

  it("getSubmodelById", function() {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmodelById(map.getId()));
  });

  it("getSubmodelById (invalid)", function() {
    var map = helper.createCustomMap();
    assert.equal(map.getSubmodelById(-1), null);
  });

  it("getSubmodelById (string id)", function() {
    var map = helper.createCustomMap();
    assert.ok(map.getSubmodelById(map.getId() + ""));
  });

  it("customizeGoogleMapView 2", function() {
    var map = helper.createCustomMap();
    map.customizeGoogleMapView(true);
  });

  it("openLayoutById (not existing)", function() {
    var map = helper.createCustomMap();
    map.openLayoutById(-1);
    assert.equal(logger.getErrors().length, 1);
  });

  it("openLayout", function() {
    var options = helper.createCustomMapOptions();
    var model = options.getProject().getModel();
    var layout = model.getLayouts()[0];
    var map = new CustomMap(options);
    map.openLayoutById(layout.getId());
    assert.equal(logger.getErrors().length, 0);
  });

  it("openLayout (string id)", function() {
    var options = helper.createCustomMapOptions();
    var model = options.getProject().getModel();
    var layout = model.getLayouts()[0];
    var map = new CustomMap(options);
    map.openLayoutById(layout.getId() + "");
    assert.equal(logger.getErrors().length, 0);
  });

  it("addSelectedLayout", function() {
    var layoutId = 90;

    var map = helper.createCustomMap();
    
    logger.debug("YYY");
    return map.addSelectedLayout(layoutId, "test layout").then(function() {
      logger.debug("XXX");
      var vLayouts = JSON.parse(ServerConnector.getVisibleLayouts());
      assert.equal(1, vLayouts.length);
      assert.equal(layoutId, vLayouts[0]);
    }).catch(function(reason) {
      logger.debug(reason);
    });
  });

  it("_showSelectedLayout", function(done) {
    var map = helper.createCustomMap();
    var alias = helper.createAlias();
    alias.setModelId(map.getId());
    map.getModel().addAlias(alias);
    
    var reaction = helper.createReaction();
    map.getModel().addReaction(reaction);

    var layout = helper.createLayout();
    layout.setInitialized(true);
    var layoutAlias = helper.createLayoutAlias();
    layoutAlias.setId(alias.getId());
    layout.addAlias(layoutAlias);
    
    var layoutReaction = helper.createLayoutReaction();
    layoutReaction.setId(reaction.getId());
    layout.addReaction(layoutReaction);
    
    map.getModel().addLayout(layout);

    map.addSelectedLayout(layout.getId());

    map._showSelectedLayout(layout.getId(), 0, 1, function(){
      
      assert.equal(2, map.selectedLayoutOverlays[layout.getId()].length);
      assert.ok(map.selectedLayoutOverlays[layout.getId()][0] instanceof AliasOverlay);
      assert.ok(map.selectedLayoutOverlays[layout.getId()][1] instanceof ReactionOverlay);

      // now hide the layout
      map._hideSelectedLayout(layout.getId());
      assert.ok(map.selectedLayoutOverlays[layout.getId()]);
      assert.equal(0, map.selectedLayoutOverlays[layout.getId()].length);
      
      done();
    });


  });
  it("openInfoWindowForAlias", function(done) {
    var map = helper.createCustomMap();
    var layout = helper.createLayout();
    var alias = helper.createAlias();
    alias.setModelId(map.getId());
    var layoutAlias = helper.createLayoutAlias();
    layoutAlias.setId(alias.getId());
    
    layout.addAlias(layoutAlias);

    // create layout
    map.getModel().addLayout(layout);
    map.getModel().addAlias(alias);

    map.openInfoWindowForAlias(alias.getId(), map.getId(), function() {
      assert.ok(map.getAliasInfoWindowById(alias.getId()));
      assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());
      done();
    });
  });

  it("openInfoWindowForMarker", function() {
    var ovName = " overlay name";

    var onSendOverlayDetailDataRequest = 0;
    ServerConnector.addListener("onSendOverlayDetailDataRequest", function() {
      onSendOverlayDetailDataRequest++;
    });
    
    var alias = helper.createAlias();

    var map = helper.createCustomMap();
    alias.setModelId(map.getId());
    map.getModel().addAlias(alias);

    var oc = new OverlayCollection(map, ovName, true, false);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));
    oc.setResultsIds([ 'result_id' ]);

    map.renderOverlayCollection(oc);

    var aliasMarker = new AliasMarker(alias.getId(), "marker/empty.png", alias, map);

    map.openInfoWindowForMarker(aliasMarker);

    assert.equal(1, onSendOverlayDetailDataRequest);

    assert.ok(map.getAliasInfoWindowById(alias.getId()));
    assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());

  });

  it("updateOverviewElementDetailData in OverlayCollection", function(done) {
    var alias = helper.createAlias();
    var ovName = " overlay name";
    var searchResultId = 'result_id';

    var map = helper.createCustomMap();

    alias.setModelId(map.getModel().getId());
    map.getModel().addAlias(alias);

    var oc = new OverlayCollection(map, ovName, true, false);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));
    oc.setResultsIds([ searchResultId ]);

    map.renderOverlayCollection({overlayCollection:oc, onready: function(){
      var aliasMarker = new AliasMarker(alias.getId(), "marker/empty.png", alias, map);
      alias.setType("RNA");

      map.openInfoWindowForMarker(aliasMarker);

      assert.ok(map.getAliasInfoWindowById(alias.getId()));
      assert.ok(map.getAliasInfoWindowById(alias.getId()).isOpened());

      var obj = {
        objectId : alias.getId(),
        modelId : map.getId(),
        type : "ALIAS"
      };

      var responseValue = "responseVVV";
      oc.updateOverviewElementDetailData(obj, searchResultId, {
        field : responseValue
      });

      assert.ok(map.getAliasInfoWindowById(alias.getId()).getContent().innerHTML.indexOf(responseValue) >= 0);
      assert.ok(map.getAliasInfoWindowById(alias.getId()).getContent().innerHTML.indexOf(searchResultId) >= 0);
      done();
      }}
   );


  });

  it("registerSource 1", function() {
    var tmp = null;
    var map = helper.createCustomMap();

    map.registerSource = function(overlayCollection) {
      tmp = overlayCollection;
    };
    var oc = new OverlayCollection(map, 'testOV');
    assert.equal(tmp, oc);
  });

  it("registerSource 2", function() {
    var map = helper.createCustomMap();

    var overlays1 = 0;
    for ( var key1 in map.overlayCollections) {
      if (map.overlayCollections.hasOwnProperty(key1) && map.overlayCollections[key1] !== null
          && map.overlayCollections[key1] !== undefined) {
        overlays1++;
      }
    }
    assert.equal(0, overlays1);
    new OverlayCollection(map, 'testOV');

    var overlays2 = 0;
    for ( var key2 in map.overlayCollections) {
      if (map.overlayCollections.hasOwnProperty(key2) && map.overlayCollections[key2] !== null
          && map.overlayCollections[key2] !== undefined) {
        overlays2++;
      }
    }
    assert.equal(1, overlays2);

  });

  it("renderOverlayCollection for alias", function(done) {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction();
    var alias = helper.createAlias();
    map.getModel().addAlias(alias);
    map.getModel().addReaction(reaction);

    var oc = new OverlayCollection(map, 'name');

    oc.elements.push(new IdentifiedElement({
      objectId : alias.getId(),
      icon : "empty.png",
      modelId : map.getId(),
      type : "Alias"
    }));

    map.renderOverlayCollection({overlayCollection:oc, onready: function(){
      var aliasMarkerCount = 0;
      for ( var id in oc.aliasMarkers) {
        if (oc.aliasMarkers.hasOwnProperty(id)) {
          aliasMarkerCount++;
        }
      }
      
      assert.equal(1, aliasMarkerCount);
      done();
   }});


  });

  it("renderOverlayCollection for point", function(done) {
    var map = helper.createCustomMap();

    var oc = new OverlayCollection(map, 'name');

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));

    map.renderOverlayCollection({overlayCollection:oc, onready: function(){
      var markerCount = 0;
      for ( var id in oc.pointMarkers) {
        if (oc.pointMarkers.hasOwnProperty(id)) {
          markerCount++;
        }
      }
      assert.equal(1, markerCount);
      done();
    }
    });
  });

  it("renderOverlayCollection for reaction", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction();
    map.getModel().addReaction(reaction);

    var oc = new OverlayCollection(map, 'name');

    var javaObj = {
      objectId : reaction.getId(),
      modelId : map.getId(),
      type : "Reaction",
    };
    

    oc.elements.push(new IdentifiedElement(javaObj));

    map.renderOverlayCollection({overlayCollection:oc, onready: function(){
      var markerCount = 0;
      for ( var id in oc.reactionMarkers) {
        if (oc.reactionMarkers.hasOwnProperty(id)) {
          markerCount++;
        }
      }

      assert.equal(1, markerCount);
    }});

  });

  it("clearData", function() {
    var map = helper.createCustomMap();

    var ovName = 'sopme name';

    var oc = new OverlayCollection(map, ovName);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));

    map.renderOverlayCollection(oc);

    var onSendClearRequest = 0;
    ServerConnector.addListener("onSendClearRequest", function(e) {
      assert.equal(ovName, e.param);
      onSendClearRequest++;
    });

    map.clearData();

    var markerCount = 0;
    for ( var id in oc.pointMarkers) {
      if (oc.pointMarkers.hasOwnProperty(id)) {
        markerCount++;
      }
    }

    assert.equal(0, markerCount);
    assert.equal(1, onSendClearRequest);

  });

  it("getInfoWindowForIdentifiedElement ( reaction)", function() {
    var reactionId = 8;
    var map = helper.createCustomMap();

    var javaObj = {
      objectId : reactionId,
      modelId : map.getId(),
      type : "REACTION"
    };

    var ie = new IdentifiedElement(javaObj);
    var infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.equal(null, infoWindow);
    map._openInfoWindowForReaction(reactionId);

    infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.ok(infoWindow);
  });

  it("right click on map", function() {
    var map = helper.createCustomMap();

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'rightclick', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });
  it("left click on map", function() {
    var map = helper.createCustomMap();

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'click', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });

  it("create polygon through drawingManager", function() {
    var map = helper.createCustomMap();

    map.turnOnDrawing();

    var triangleCoords = [ new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
        new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190), ];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths : triangleCoords,
      strokeColor : '#FF0000',
      strokeOpacity : 0.8,
      strokeWeight : 2,
      fillColor : '#FF0000',
      fillOpacity : 0.35
    });
    bermudaTriangle.setMap(map.getGoogleMap());

    var eventParam = {
      type : google.maps.drawing.OverlayType.POLYGON,
      overlay : bermudaTriangle,
    };

    // check creation complete behaviour
    assert.ok(map.isDrawingOn());
    assert.equal(google.maps.drawing.OverlayType.POLYGON, map._drawingManager.getDrawingMode());
    google.maps.event.trigger(map._drawingManager, 'overlaycomplete', eventParam);
    assert.ok(map.isDrawingOn());
    assert.equal(null, map._drawingManager.getDrawingMode());

    // and now check how it behaves when we wiught click on the polygon
    var clickEventParam = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(ServerConnector.getSelectedPolygon());
    assert.notOk(ServerConnector.getExportModelId());

    assert.notOk(map.getActiveSubmapId());
    assert.notOk(map.getSelectedArea());
    google.maps.event.trigger(bermudaTriangle, 'rightclick', clickEventParam);
    assert.ok(map.getSelectedArea());
    assert.equal(map.getId(), map.getActiveSubmapId());

    assert.ok(ServerConnector.getSelectedPolygon());
    assert.ok(ServerConnector.getExportModelId());

  });
});
